/***************************************************************************
 *   Copyright (C) 2010 by Conrad Hoffmann                                 *
 *   ch@bitfehler.net                                                      *
 *   http://code.bitfehler.net                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2,       *
 *   as published by the Free Software Foundation,                         *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "check.h"

#include <KDebug>
#include <KConfigGroup>

#include "pingcheck.h"
#include "socketcheck.h"
#include "commandcheck.h"
#include "serverstatus.h"
#include "ui_pingchecksettings.h"
#include "ui_socketchecksettings.h"
#include "ui_commandchecksettings.h"


namespace ServerStatus
{

class CheckStatic
{
public:
	Ui::PingCheckSettings pingCheckSettings;
	Ui::SocketCheckSettings socketCheckSettings;
	Ui::CommandCheckSettings commandCheckSettings;
};

} // end namespace ServerStatus

using namespace ServerStatus;

CheckStatic* Check::m_static = new CheckStatic;
QStringList Check::m_checkTypes = QStringList();

const QString Check::TypePing = QString("Ping");
const QString Check::TypeSocket = QString("TCP connection");
const QString Check::TypeCommand = QString("Run command");

// static
const QStringList& Check::checkTypes()
{
	if (m_checkTypes.empty()) {
		m_checkTypes << Check::TypePing << Check::TypeSocket << Check::TypeCommand;
	}
	return m_checkTypes;
}

// static
Check* Check::createCheck(const QString& type)
{
	if (type == Check::TypePing) {
		Check* result = new PingCheck();
		return result;
	}
	else if (type == Check::TypeSocket) {
		Check* result = new SocketCheck();
		return result;
	}
	else if (type == Check::TypeCommand) {
		Check* result = new CommandCheck();
		return result;
	}
	kDebug(0) << "Cannot create check of requested type" << type;
	return 0L;
}

// static
void Check::showSettings(QWidget* w, const QString& type, const Check* const currentCheck)
{
	if (type == Check::TypePing) {
		PingCheck::showSettings(w, currentCheck);
	}
	else if (type == Check::TypeSocket) {
		SocketCheck::showSettings(w, currentCheck);
	}
	else if (type == Check::TypeCommand) {
		CommandCheck::showSettings(w, currentCheck);
	}
}

// static
void Check::cleanup()
{
	delete m_static;
}

Check::Check(const QString& type) : QThread(),
	m_type(type)
{
}

void Check::slotFinished()
{
	const Status& s = result();
	emit(finished(s));
}


#include "check.moc"
