/***************************************************************************
 *   Copyright (C) 2010 by Conrad Hoffmann                                 *
 *   ch@bitfehler.net                                                      *
 *   http://code.bitfehler.net                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2,       *
 *   as published by the Free Software Foundation,                         *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef SERVERSTATUS_CHECK_H
#define SERVERSTATUS_CHECK_H

#include <QThread>
#include <QString>
#include <QStringList>

class KConfigGroup;

namespace ServerStatus
{

class Status;
class CheckStatic;

class Check : public QThread
{

Q_OBJECT

public:
	
	static const QString TypePing;
	static const QString TypeSocket;
	static const QString TypeCommand;
	
	static const QStringList& checkTypes();
	
	static Check* createCheck(const QString& type);
	
	static void showSettings(QWidget* w, const QString& type, const Check* const currentCheck);
	
	static void cleanup();
	
	inline const QString& type() const { return m_type; };

	virtual void applySettings() = 0;
	
	virtual QStringList parameters() const = 0;
	
	virtual void setParameters(const QStringList& parameters) = 0;
	
	virtual const Status& result() = 0;
	
Q_SIGNALS:

	void finished(const ServerStatus::Status& result);
	
protected:
	
	Check(const QString& type);
	
	static CheckStatic* m_static;

protected Q_SLOTS:
	
	void slotFinished();
	
private:
	
	QString m_type;
	
	static QStringList m_checkTypes;

};

} // end namespace serverstatus

#endif

