/***************************************************************************
 *   Copyright (C) 2010 by Conrad Hoffmann                                 *
 *   ch@bitfehler.net                                                      *
 *   http://code.bitfehler.net                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2,       *
 *   as published by the Free Software Foundation,                         *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "commandcheck.h"

#include <QProcess>

#include <KDebug>

#include "serverstatus.h"

namespace ServerStatus
{

class CommandCheckPrivate
{
public:
	QString command;

	static Status statusUp;
	static Status statusDown;
	static Status statusError;
	
	int result;
};

} // end namespace ServerStatus


using namespace ServerStatus;

Ui::CommandCheckSettings CommandCheck::m_settingsUi;

Status CommandCheckPrivate::statusUp(Status::Up, 
									 i18nc("server status", "up"), 
									 i18n("The server is up."));
Status CommandCheckPrivate::statusDown(Status::Down, 
									   i18nc("server status", "down"),
									   i18n("The server is down!"));
Status CommandCheckPrivate::statusError(Status::Error, 
										i18nc("server status", "check failed"), 
										i18n("Failed to execute the given command."));

CommandCheck::CommandCheck() : Check(Check::TypeCommand)
{
	d = new CommandCheckPrivate;
	d->result = 0;
}

CommandCheck::~CommandCheck()
{
	delete d;
}

// static
void CommandCheck::showSettings(QWidget* w, const Check* const currentCheck)
{
	CommandCheck::m_settingsUi.setupUi(w);
	if (currentCheck != 0L && currentCheck->type() == Check::TypeCommand) {
		QStringList params = currentCheck->parameters();
		CommandCheck::m_settingsUi.leCommand->setText(params.at(0));
	}
}

void CommandCheck::applySettings()
{
	d->command = CommandCheck::m_settingsUi.leCommand->text().trimmed();
}

QStringList CommandCheck::parameters() const
{
	QStringList result;
	result << d->command;
	return result;
}

void CommandCheck::setParameters(const QStringList& parameters)
{
	if (parameters.size() == 1) {
		d->command = parameters.at(0);
	}
}

const Status& CommandCheck::result()
{
	switch (d->result) {
	case 1:
		return CommandCheckPrivate::statusUp;
	case 2:
		return CommandCheckPrivate::statusDown;
	case 3:
		return CommandCheckPrivate::statusError;
	default:
		return Status::None;
	}
}

void CommandCheck::run()
{
	kDebug(0) << "Starting new thread for command check";
	QProcess proc;
	
	proc.start(d->command);
	proc.waitForFinished();
	
	if (proc.exitStatus() == QProcess::CrashExit) {
		d->result = 3;
		return;
	}

	switch (proc.exitCode()) {
	case 0:
		// Success means the server is up.
		d->result = 1;
		return;
	default:
		// Everything else counts as down.
		d->result = 2;
		return;
	}
}

#include "commandcheck.moc"
