/***************************************************************************
 *   Copyright (C) 2010 by Conrad Hoffmann                                 *
 *   ch@bitfehler.net                                                      *
 *   http://code.bitfehler.net                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2,       *
 *   as published by the Free Software Foundation,                         *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef SERVERSTATUS_SERVER_H
#define SERVERSTATUS_SERVER_H

#include <QObject>

class QString;

class KConfigGroup;

namespace ServerStatus
{

class Check;
class Status;
class ServerPrivate;

class Server : public QObject
{

Q_OBJECT
	
public:

	Server(const QString& id);
	Server(const QString& id, KConfigGroup& config);
	
	const QString& id() const { return m_id; };

	void readConfig(KConfigGroup& config);
	void saveConfig(KConfigGroup& config);
	void removeConfig(KConfigGroup& config);
	
	bool checksRunning();
	
	void startChecks();
	
	void stopChecks();
	
	bool active() const;
	
	void setActive(bool active);

	const QString& name() const;
	
	void setName(const QString& name);
	
	int interval() const;
	
	void setInterval(int interval);
	
	Check* check();
	
	void setCheck(Check* check);

	void delayedDeletion();

	void forcedDeletion();
	
Q_SIGNALS:
	
	void statusChanged(const ServerStatus::Status& status);
	
	void statusChanged(const ServerStatus::Server* server, const ServerStatus::Status& status);

private Q_SLOTS:

	void slotCheckFinished();
	void slotTimeout();

private:

	~Server();

	ServerPrivate* d;
	
	QString m_id;
	QString m_cfgTemplate;
};

} // end namespace serverstatus

#endif

