/***************************************************************************
 *   Copyright (C) 2010 by Conrad Hoffmann                                 *
 *   ch@bitfehler.net                                                      *
 *   http://code.bitfehler.net                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2,       *
 *   as published by the Free Software Foundation,                         *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef SERVERSTATUSCONFIG_H
#define SERVERSTATUSCONFIG_H

#include <QObject>
#include <QStringList>

class QFont;
class KConfigDialog;
class KConfigGroup;

namespace ServerStatus
{

class Status;
class Server;
class ConfigPrivate;

class Config : public QObject
{

Q_OBJECT

public:

	Config(QObject* parent);
	~Config();
	
	void loadConfig(KConfigGroup& config);
	void saveConfig(KConfigGroup& config);
	
	void createConfigurationInterface(KConfigDialog *parent);
	
	const QList<Server*>& servers() const;
	
	const QString &iconForStatus(const Status &status) const;
	
	const QString& iconOk();
	const QString& iconError();
	const QString& iconWarning();
	const QString& iconUnknown();
	
	const QFont& serverFont();

Q_SIGNALS:
	
	void configChanged();

private:
	
	void reloadServerList();
	
	ConfigPrivate *d;
	
private Q_SLOTS:

	void slotServerListSelectionChanged();
	void slotCheckTypeSelectionChanged(const QString& name);
	
	void slotAddServer();
	void slotEditServer();
	void slotRemoveServer();
	
	void slotMoveServerUp();
	void slotMoveServerDown();
	
	void slotSelectIconOk();
	void slotSelectIconError();
	void slotSelectIconWarning();
	void slotSelectIconUnknown();
	void slotSelectServerFont();
	
	void slotConfigAccepted();
	void slotConfigCancelled();

};

} // end namespace ServerStatus


#endif

