/***************************************************************************
 *   Copyright (C) 2010 by Conrad Hoffmann                                 *
 *   ch@bitfehler.net                                                      *
 *   http://code.bitfehler.net                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2,       *
 *   as published by the Free Software Foundation,                         *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "socketcheck.h"

#include <inttypes.h>

#include <QRegExp>
#include <QTcpSocket>

#include <KDebug>

#include "serverstatus.h"


namespace ServerStatus
{

class SocketCheckPrivate
{
public:
	uint16_t port;
	QString host;

	static Status statusUp;
	static Status statusDown;
	static Status statusError;
	
	int result;
};

} // end namespace ServerStatus


using namespace ServerStatus;

Ui::SocketCheckSettings SocketCheck::m_settingsUi;

Status SocketCheckPrivate::statusUp(Status::Up, 
									i18nc("server status", "up"), 
									i18n("The server is up."));
Status SocketCheckPrivate::statusDown(Status::Down, 
									  i18nc("server status", "down"), 
									  i18n("Connection to server failed!"));
Status SocketCheckPrivate::statusError(Status::Error, 
									   i18nc("server status", "check failed"), 
									   i18n("Local error during connection attempt."));

SocketCheck::SocketCheck() : Check(Check::TypeSocket)
{
	d = new SocketCheckPrivate;
	d->result = 0;
}

SocketCheck::~SocketCheck()
{
	delete d;
}

// static
void SocketCheck::showSettings(QWidget* w, const Check* const currentCheck)
{
	SocketCheck::m_settingsUi.setupUi(w);
	if (currentCheck != 0L && currentCheck->type() == Check::TypeSocket) {
		QStringList params = currentCheck->parameters();
		SocketCheck::m_settingsUi.leHost->setText(params.at(0));
		SocketCheck::m_settingsUi.sbPort->setValue(params.at(1).toUInt());
	}
}

void SocketCheck::applySettings()
{
	d->host = SocketCheck::m_settingsUi.leHost->text();
	d->port = SocketCheck::m_settingsUi.sbPort->value();
}

QStringList SocketCheck::parameters() const
{
	QStringList result;
	result << d->host;
	result << QString::number(d->port);
	return result;
}

void SocketCheck::setParameters(const QStringList& parameters)
{
	d->host = parameters.at(0);
	d->port = parameters.at(1).toUShort();
}

const Status& SocketCheck::result()
{
	switch (d->result) {
	case 1:
		return SocketCheckPrivate::statusUp;
	case 2:
		return SocketCheckPrivate::statusDown;
	case 3:
		return SocketCheckPrivate::statusError;
	default:
		return Status::None;
	}
}

void SocketCheck::run()
{
	kDebug(0) << "Starting new Thread for connection check";
	QTcpSocket socket;
	socket.connectToHost(d->host, d->port);
	if (socket.waitForConnected(1000)) {
		socket.disconnectFromHost();
		d->result = 1;
		return;
	}

	// Check, wheter a local error or a connection error occured.
	switch (socket.error()) {
	// The following three errors are probably the only ones that indicate
	// that the host or service is down.
	case QAbstractSocket::ConnectionRefusedError:
	case QAbstractSocket::HostNotFoundError:
	case QAbstractSocket::SocketTimeoutError:
		socket.disconnectFromHost();
		d->result = 2;
		return;
	default:
		socket.disconnectFromHost();
		d->result = 3;
		return;
	}
}


#include "socketcheck.moc"
